@@ -1,10 +1,9 @@
-#!/usr/bin/env python
 # encoding: utf-8
 """
 configuration.py
 
 Created by Thomas Mangin on 2009-08-25.
-Copyright (c) 2009-2011 Exa Networks. All rights reserved.
+Copyright (c) 2009-2012 Exa Networks. All rights reserved.
 """
 
 import os
@@ -20,16 +19,16 @@ from exabgp.structure.icmp       import NamedICMPType,NamedICMPCode
 from exabgp.structure.tcpflags   import NamedTCPFlags
 from exabgp.structure.fragments  import NamedFragments
 from exabgp.message.open         import HoldTime,RouterID
-from exabgp.message.update.route import Route
+#from exabgp.message.update.route import Route
 from exabgp.message.update.flow  import BinaryOperator,NumericOperator
 from exabgp.message.update.flow  import Flow,Source,Destination,SourcePort,DestinationPort,AnyPort,IPProtocol,TCPFlag,Fragment,PacketLength,ICMPType,ICMPCode,DSCP
-from exabgp.message.update.attribute             import AttributeID,Attribute
+from exabgp.message.update.attribute             import AttributeID #,Attribute
 from exabgp.message.update.attribute.origin      import Origin
 from exabgp.message.update.attribute.nexthop     import NextHop
 from exabgp.message.update.attribute.aspath      import ASPath
 from exabgp.message.update.attribute.med         import MED
 from exabgp.message.update.attribute.localpref   import LocalPreference
-from exabgp.message.update.attribute.communities import Community,Communities,to_FlowTrafficRate,to_RouteTargetCommunity_00,to_RouteTargetCommunity_01
+from exabgp.message.update.attribute.communities import Community,Communities,ECommunity,ECommunities,to_ExtendedCommunity,to_FlowTrafficRate,to_RouteTargetCommunity_00,to_RouteTargetCommunity_01
 
 from exabgp.log import Logger
 logger = Logger()
@@ -48,13 +47,18 @@ class Watchdog (str):
 	ID = AttributeID.INTERNAL_WATCHDOG
 	MULTIPLE = False
 
+class Withdrawn (object):
+	ID = AttributeID.INTERNAL_WITHDRAWN
+	MULTIPLE = False
 
 class Configuration (object):
 	TTL_SECURITY = 255
 	debug = os.environ.get('RAISE_CONFIGURATION',None) != None
 
 	_str_route_error = \
-	'community and as-path can take a single community as parameter. only next-hop is mandatory\n\n' \
+	'community, extended-communities and as-path can take a single community as parameter.\n' \
+	'only next-hop is mandatory\n' \
+	'\n' \
 	'syntax:\n' \
 	'route 10.0.0.1/22 {\n' \
 	'  next-hop 192.0.1.254;\n' \
@@ -63,10 +67,13 @@ class Configuration (object):
 	'  med 100;\n' \
 	'  local-preference 100;\n' \
 	'  community [ 65000 65001 65002 ];\n' \
+	'  extended-community [ target:1234:5.6.7.8 target:1.2.3.4:5678 origin:1234:5.6.7.8 origin:1.2.3.4:5678 0x0002FDE800000001 ]\n' \
 	'  label [ 100 200 ];\n' \
 	'  split /24\n' \
 	'  watchdog watchog-name\n' \
-	'}\n\n' \
+	'  withdrawn\n' \
+	'}\n' \
+	'\n' \
 	'syntax:\n' \
 	'route 10.0.0.1/22 next-hop 192.0.2.1' \
 	' origin IGP|EGP|INCOMPLETE' \
@@ -110,7 +117,7 @@ class Configuration (object):
 		self._text = text
 		self._fname = fname
 		self._clear()
-	
+
 	def _clear (self):
 		self.process = {}
 		self.neighbor = {}
@@ -168,7 +175,7 @@ class Configuration (object):
 	def parse_single_flow (self,command):
 		self._tokens = self._tokenise(' '.join(command.split(' ')[2:]).split('\\n'))
 		scope = [{}]
-		if not self._dispatch(scope,'flow',['route',],[]): 
+		if not self._dispatch(scope,'flow',['route',],[]):
 			return None
 		if not self._check_flow_route(scope):
 			return None
@@ -327,13 +334,16 @@ class Configuration (object):
 		if command == 'route': return self._single_static_route(scope,tokens[1:])
 		if command == 'origin': return self._route_origin(scope,tokens[1:])
 		if command == 'as-path': return self._route_aspath(scope,tokens[1:])
+		if command == 'as-sequence': return self._route_aspath(scope,tokens[1:])
 		if command == 'med': return self._route_med(scope,tokens[1:])
 		if command == 'next-hop': return self._route_next_hop(scope,tokens[1:])
 		if command == 'local-preference': return self._route_local_preference(scope,tokens[1:])
 		if command == 'community': return self._route_community(scope,tokens[1:])
 		if command == 'split': return self._route_split(scope,tokens[1:])
 		if command == 'label': return self._route_label(scope,tokens[1:])
-		
+		if command == 'watchdog': return self._route_watchdog(scope,tokens[1:])
+		if command == 'withdrawn': return self._route_withdrawn(scope,tokens[1:])
+
 		if command == 'source': return self._flow_source(scope,tokens[1:])
 		if command == 'destination': return self._flow_destination(scope,tokens[1:])
 		if command == 'port': return self._flow_route_anyport(scope,tokens[1:])
@@ -356,7 +366,7 @@ class Configuration (object):
 		return False
 
 	# Group Watchdog
-	
+
 	def _multi_process (self,scope,tokens):
 		if len(tokens) != 1:
 			self._error = self._str_process_error
@@ -374,15 +384,26 @@ class Configuration (object):
 		return True
 
 	def _set_process_run (self,scope,command,value):
-		prg = value[0]
-		if len(prg) > 2 and prg[0] == prg[-1] and prg[0] in ['"',"'"]:
-			prg = prg[1:-1]
+		line = ' '.join(value).strip()
+		if len(line) > 2 and line[0] == line[-1] and line[0] in ['"',"'"]:
+			line = line[1:-1]
+		if ' ' in line:
+			prg,args = line.split(' ',1)
+		else:
+			prg = line
+			args = ''
+
 		if not prg:
 			self._error = 'prg requires the program to prg as an argument (quoted or unquoted)'
 			if self.debug: raise
 			return False
 		if prg[0] != '/':
-			prg = os.path.join(os.getcwd(),prg)
+			if prg.startswith('etc/exabgp'):
+				parts = prg.split('/')
+				path = [os.environ.get('ETC','etc'),] + parts[2:]
+				prg = os.path.join(*path)
+			else:
+				prg = os.path.join(os.getcwd(),prg)
 		if not os.path.exists(prg):
 			self._error = 'can not locate the the program "%s"' % prg
 			if self.debug: raise
@@ -390,7 +411,7 @@ class Configuration (object):
 
 		# XXX: Yep, race conditions are possible, those are sanity checks not security ones ...
 		s = os.stat(prg)
-		
+
 		if stat.S_ISDIR(s.st_mode):
 			self._error = 'can not execute directories "%s"' % prg
 			if self.debug: raise
@@ -412,7 +433,10 @@ class Configuration (object):
 			if self.debug: raise
 			return False
 
-		scope[-1][command] = prg
+		if args:
+			scope[-1][command] = [prg] + args.split(' ')
+		else:
+			scope[-1][command] = [prg,]
 		return True
 
 	def _route_watchdog (self,scope,tokens):
@@ -424,6 +448,15 @@ class Configuration (object):
 			if self.debug: raise
 			return False
 
+	def _route_withdrawn (self,scope,tokens):
+		try:
+			scope[-1]['routes'][-1].attributes.add(Withdrawn())
+			return True
+		except ValueError:
+			self._error = self._str_route_error
+			if self.debug: raise
+			return False
+
 	# Group Neighbor
 
 	def _multi_group (self,scope,address):
@@ -488,11 +521,11 @@ class Configuration (object):
 			return False
 		if neighbor.local_address.afi != neighbor.peer_address.afi:
 			self._error = 'local-address and peer-address must be of the same family'
-			return False 
+			return False
 		if self._neighbor.has_key(neighbor.peer_address.ip):
 			self._error = 'duplicate peer definition %s' % neighbor.peer_address.ip
 			return False
-		
+
 		self._neighbor[neighbor.peer_address.ip] = neighbor
 		return True
 
@@ -646,7 +679,7 @@ class Configuration (object):
 
 		# ignore if the request is for an aggregate, or the same size
 		mask = route.nlri.mask
-		split = route[AttributeID.INTERNAL_SPLIT]
+		split = route.attributes[AttributeID.INTERNAL_SPLIT]
 		if mask >= split:
 			return True
 
@@ -677,15 +710,15 @@ class Configuration (object):
 				lower = ipn&0xFF
 				ipn = ipn >> 8
 				i = chr(lower) + i
-			
+
 			# change the route network
 			r.nlri.update_raw(i)
 			# update ip to the next route
 			ip += increment
-			
+
 			# save route
 			scope[-1]['routes'].append(r)
-		
+
 		# route is no longer needed - delete it explicitely
 		del(route)
 		return True
@@ -742,7 +775,7 @@ class Configuration (object):
 			return False
 
 		while True:
-			r = self._dispatch(scope,'route',[],['next-hop','origin','as-path','med','local-preference','community','split','label'])
+			r = self._dispatch(scope,'route',[],['next-hop','origin','as-path','as-sequence','med','local-preference','community','extended-community','split','label','watchdog','withdrawn'])
 			if r is False: return False
 			if r is None: return self._split_last_route(scope)
 
@@ -760,9 +793,15 @@ class Configuration (object):
 			return False
 
 		while len(tokens):
-			if len(tokens) < 2:
-				return False
 			command = tokens.pop(0)
+			if command == 'withdrawn':
+				if self._route_withdrawn(scope,tokens):
+					continue
+				return False
+
+			if len(tokens) < 1:
+				return False
+
 			if command == 'origin':
 				if self._route_origin(scope,tokens):
 					continue
@@ -771,6 +810,10 @@ class Configuration (object):
 				if self._route_aspath(scope,tokens):
 					continue
 				return False
+			if command == 'as-sequence':
+				if self._route_aspath(scope,tokens):
+					continue
+				return False
 			if command == 'med':
 				if self._route_med(scope,tokens):
 					continue
@@ -783,6 +826,10 @@ class Configuration (object):
 				if self._route_community(scope,tokens):
 					continue
 				return False
+			if command == 'extended-community':
+				if self._route_extended_community(scope,tokens):
+					continue
+				return False
 			if command == 'split':
 				if self._route_split(scope,tokens):
 					continue
@@ -872,19 +919,26 @@ class Configuration (object):
 			prefix = int(data[:separator])
 			suffix = int(data[separator+1:])
 			if prefix >= pow(2,16):
-				raise ValueError('invalid community %s (prefix too large)' % data) 
+				raise ValueError('invalid community %s (prefix too large)' % data)
 			if suffix >= pow(2,16):
-				raise ValueError('invalid community %s (suffix too large)' % data) 
+				raise ValueError('invalid community %s (suffix too large)' % data)
 			return Community((prefix<<16) + suffix)
 		elif len(data) >=2 and data[1] in 'xX':
 			value = long(data,16)
 			if value >= pow(2,32):
-				raise ValueError('invalid community %s (too large)' % data) 
+				raise ValueError('invalid community %s (too large)' % data)
 			return Community(value)
 		else:
+			low = data.lower()
+			if low == 'no-export':
+				data = 0xFFFFFF01
+			elif low == 'no-advertise':
+				data = 0xFFFFFF02
+			elif low == 'no-export-subconfed':
+				data = 0xFFFFFF03
 			value = long(data)
 			if value >= pow(2,32):
-				raise ValueError('invalid community %s (too large)' % data) 
+				raise ValueError('invalid community %s (too large)' % data)
 			return Community(value)
 
 	def _route_community (self,scope,tokens):
@@ -911,6 +965,47 @@ class Configuration (object):
 		scope[-1]['routes'][-1].attributes.add(communities)
 		return True
 
+	def _parse_extended_community (self,scope,data):
+		if data[:2].lower() == '0x':
+			try:
+				raw = ''
+				for i in range(2,len(data),2):
+					raw += chr(int(data[i:i+2],16))
+			except ValueError:
+				raise ValueError('invalid extended community %s' % data)
+			if len(raw) != 8:
+				raise ValueError('invalid extended community %s' % data)
+			return ECommunity(raw)
+		elif data.count(':'):
+			return to_ExtendedCommunity(data)
+		else:
+			raise ValueError('invalid extended community %s - lc+gc' % data)
+
+	def _route_extended_community (self,scope,tokens):
+		extended_communities = ECommunities()
+		extended_community = tokens.pop(0)
+		try:
+			if extended_community == '[':
+				while True:
+					try:
+						extended_community = tokens.pop(0)
+					except IndexError:
+						self._error = self._str_route_error
+						if self.debug: raise
+						return False
+					if extended_community == ']':
+						break
+					extended_communities.add(self._parse_extended_community(scope,extended_community))
+			else:
+				extended_communities.add(self._parse_extended_community(scope,extended_community))
+		except ValueError:
+			self._error = self._str_route_error
+			if self.debug: raise
+			return False
+		scope[-1]['routes'][-1].attributes.add(extended_communities)
+		return True
+
+
 	def _route_split (self,scope,tokens):
 		try:
 			size = tokens.pop(0)
@@ -992,7 +1087,7 @@ class Configuration (object):
 		return r
 
 	# ..........................................
-	
+
 	def _multi_match (self,scope,tokens):
 		if len(tokens) != 0:
 			self._error = self._str_flow_error
@@ -1156,7 +1251,7 @@ class Configuration (object):
 
 	def _flow_route_fragment (self,scope,tokens):
 		return self._flow_generic_list(scope,tokens,NamedFragments,Fragment)
-	
+
 	def _flow_route_dscp (self,scope,tokens):
 		return self._flow_generic_condition(scope,tokens,int,DSCP)
 
@@ -1214,4 +1309,4 @@ class Configuration (object):
 			self._error = self._str_route_error
 			if self.debug: raise
 			return False
-		
+
