@@ -1,10 +1,9 @@
-#!/usr/bin/env python
 # encoding: utf-8
 """
 configuration.py
 
 Created by Thomas Mangin on 2011-03-29.
-Copyright (c) 2011-2011 Exa Networks. All rights reserved.
+Copyright (c) 2011-2012 Exa Networks. All rights reserved.
 """
 
 import os
@@ -14,7 +13,7 @@ debug = os.environ.get('PDB',None)
 
 def bug_report (type, value, trace):
 	import traceback
-	from proxy.log import Logger
+	from exabgp.log import Logger
 	logger = Logger()
 
 	print
@@ -25,6 +24,12 @@ def bug_report (type, value, trace):
 	print "-"*80
 	print
 	print
+	print '-- Version'
+	print
+	print
+	print sys.version
+	print
+	print
 	print "-- Configuration"
 	print
 	print
@@ -49,19 +54,19 @@ def bug_report (type, value, trace):
 	print "-"*80
 	print
 	print
-	
+
 	#print >> sys.stderr, 'the program failed with message :', value
 
 if debug is None:
-	def intercept (type, value, trace):
+	def intercept_nopdb (type, value, trace):
 		bug_report(type, value, trace)
-	sys.excepthook = intercept
+	sys.excepthook = intercept_nopdb
 elif debug not in ['0','']:
-	def intercept (type, value, trace):
+	def intercept_pdb (type, value, trace):
 		bug_report(type, value, trace)
 		import pdb
 		pdb.pm()
-	sys.excepthook = intercept
+	sys.excepthook = intercept_pdb
 
 del sys.argv[0]
 
