@@ -1,10 +1,9 @@
-#!/usr/bin/env python
 # encoding: utf-8
 """
 attribute/__init__.py
 
 Created by Thomas Mangin on 2009-11-05.
-Copyright (c) 2009-2011 Exa Networks. All rights reserved.
+Copyright (c) 2009-2012 Exa Networks. All rights reserved.
 """
 
 from struct import pack
@@ -24,9 +23,9 @@ class Attribute (object):
 			return ''
 		length = len(value)
 		if length > 0xFF:
-			flag &= Flag.EXTENDED_LENGTH
+			flag |= Flag.EXTENDED_LENGTH
 		if flag & Flag.EXTENDED_LENGTH:
-			len_value = pack('!H',length)[0]
+			len_value = pack('!H',length)
 		else:
 			len_value = chr(length)
 		return "%s%s%s%s" % (chr(flag),chr(self.ID),len_value,value)
