@@ -1,10 +1,9 @@
-#!/usr/bin/env python
 # encoding: utf-8
 """
 id.py
 
 Created by Thomas Mangin on 2010-01-19.
-Copyright (c) 2010-2011 Exa Networks. All rights reserved.
+Copyright (c) 2010-2012 Exa Networks. All rights reserved.
 """
 
 class AttributeID (int):
@@ -23,10 +22,12 @@ class AttributeID (int):
 	EXTENDED_COMMUNITY = 0x10
 	# RFC 4893
 	AS4_PATH           = 0x11
+	AS4_AGGREGATOR     = 0x12
 	# RFC 4760
 	MP_REACH_NLRI      = 0x0e # 14
 	MP_UNREACH_NLRI    = 0x0f # 15
 
+	INTERNAL_WITHDRAWN = 0xFFFD
 	INTERNAL_WATCHDOG  = 0xFFFE
 	INTERNAL_SPLIT     = 0xFFFF
 
