@@ -1,10 +1,9 @@
-#!/usr/bin/env python
 # encoding: utf-8
 """
 eor.py
 
 Created by Thomas Mangin on 2010-01-16.
-Copyright (c) 2010-2011 Exa Networks. All rights reserved.
+Copyright (c) 2010-2012 Exa Networks. All rights reserved.
 """
 
 from exabgp.structure.address import Address,AFI,SAFI
@@ -35,12 +34,12 @@ class EOR (object):
 
 	def eors (self,families):
 		self._announced = []
-		r = ''
+		r = []
 		for afi,safi in families:
 			if afi == AFI.ipv4 and safi in [SAFI.unicast, SAFI.multicast]:
-				r += self.ipv4()
+				r.append(self.ipv4())
 			else:
-				r += self.mp(afi,safi)
+				r.append(self.mp(afi,safi))
 			self._announced.append((afi,safi))
 		return r
 
