@@ -1,14 +1,13 @@
-#!/usr/bin/env python
 # encoding: utf-8
 """
 protocol.py
 
 Created by Thomas Mangin on 2009-08-25.
-Copyright (c) 2009-2011 Exa Networks. All rights reserved.
+Copyright (c) 2009-2012 Exa Networks. All rights reserved.
 """
 
-import os
-import copy
+#import os
+#import copy
 import time
 import socket
 from struct import unpack
@@ -21,29 +20,31 @@ from exabgp.structure.address    import AFI,SAFI
 from exabgp.structure.ip         import BGPPrefix,Inet,to_IP
 from exabgp.structure.asn        import ASN,AS_TRANS
 from exabgp.network.connection   import Connection
-from exabgp.message              import Message,defix
+from exabgp.message              import Message,defix,Failure
 from exabgp.message.nop          import NOP
-from exabgp.message.open         import Open,Unknown,Parameter,Capabilities,RouterID,MultiProtocol,RouteRefresh,CiscoRouteRefresh,Graceful
+from exabgp.message.open         import Open,Unknown,Parameter,Capabilities,RouterID,MultiProtocol,RouteRefresh,CiscoRouteRefresh,MultiSession,Graceful
 from exabgp.message.update       import Update
 from exabgp.message.update.eor   import EOR
 from exabgp.message.keepalive    import KeepAlive
-from exabgp.message.notification import Notification, Notify, NotConnected
-from exabgp.message.update.route import Route,ReceivedRoute
+from exabgp.message.notification import Notification, Notify #, NotConnected
+from exabgp.message.update.route import ReceivedRoute # ,Route
 from exabgp.message.update.attributes     import Attributes
 from exabgp.message.update.attribute      import AttributeID
 from exabgp.message.update.attribute.flag        import Flag
 from exabgp.message.update.attribute.origin      import Origin
-from exabgp.message.update.attribute.aspath      import ASPath
+from exabgp.message.update.attribute.aspath      import ASPath,AS4Path
 from exabgp.message.update.attribute.nexthop     import NextHop
 from exabgp.message.update.attribute.med         import MED
 from exabgp.message.update.attribute.localpref   import LocalPreference
-from exabgp.message.update.attribute.communities import Community,Communities
+from exabgp.message.update.attribute.communities import Community,Communities,ECommunity,ECommunities
 #from exabgp.message.update.attribute.mprnlri     import MPRNLRI
-from exabgp.message.update.attribute.mpurnlri    import MPURNLRI
+#from exabgp.message.update.attribute.mpurnlri    import MPURNLRI
 
 from exabgp.log import Logger
 logger = Logger()
 
+MAX_BACKLOG = 200000
+
 # README: Move all the old packet decoding in another file to clean up the includes here, as it is not used anyway
 
 class Protocol (object):
@@ -56,6 +57,9 @@ class Protocol (object):
 		self.connection = connection
 		self._delta = Delta(Table(peer))
 		self._asn4 = False
+		self._messages = {}
+		self._frozen = 0
+		self.message_size = 4096
 
 	def me (self,message):
 		return "Peer %15s ASN %-7s %s" % (self.peer.neighbor.peer_address,self.peer.neighbor.peer_as,message)
@@ -68,6 +72,8 @@ class Protocol (object):
 			md5 = self.neighbor.md5
 			ttl = self.neighbor.ttl
 			self.connection = Connection(peer,local,md5,ttl)
+			for name in self.peer.supervisor.processes.receive_routes():
+				self.peer.supervisor.processes.write(name,"neighbor %s up\n" % self.peer.neighbor.peer_address)
 
 	def check_keepalive (self):
 		left = int (self.connection.last_read  + self.neighbor.hold_time - time.time())
@@ -78,6 +84,8 @@ class Protocol (object):
 	def close (self):
 		#self._delta.last = 0
 		if self.connection:
+			for name in self.peer.supervisor.processes.receive_routes():
+				self.peer.supervisor.processes.write(name,"neighbor %s down\n" % self.peer.neighbor.peer_address)
 			self.connection.close()
 			self.connection = None
 
@@ -85,14 +93,17 @@ class Protocol (object):
 	# Read from network .......................................................
 
 	def read_message (self):
-		if not self.connection.pending():
+		# This call reset the time for the timeout in
+		if not self.connection.pending(True):
 			return NOP('')
 
-		data = self.connection.read(19)
-
-		# It seems that select tells us there is data even when there isn't
-		if not data:
-			raise NotConnected(self.neighbor.peer_address)
+		length = 19
+		data = ''
+		while length:
+			if self.connection.pending():
+				delta = self.connection.read(length)
+				data += delta
+				length -= len(delta)
 
 		if data[:16] != Message.MARKER:
 			# We are speaking BGP - send us a valid Marker
@@ -117,10 +128,12 @@ class Protocol (object):
 			#(msg == RouteRefresh.TYPE and length != 23)
 
 		length -= 19
-		data = self.connection.read(length)
-
-		if len(data) != length:
-			raise Notify(ord(msg),0)
+		data = ''
+		while length:
+			if self.connection.pending():
+				delta = self.connection.read(length)
+				data += delta
+				length -= len(delta)
 
 		if msg == Notification.TYPE:
 			raise Notification(ord(data[0]),ord(data[1]))
@@ -163,15 +176,26 @@ class Protocol (object):
 			peer_as = message.asn
 
 		if peer_as != self.neighbor.peer_as:
-			raise Notify(2,2,'ASN sent (%d) did not match ASN expected (%d)' % (message.asn,self.neighbor.peer_as))
+			raise Notify(2,2,'ASN in OPEN (%d) did not match ASN expected (%d)' % (message.asn,self.neighbor.peer_as))
+
+		# RFC 6286 : http://tools.ietf.org/html/rfc6286
+		#if message.router_id == RouterID('0.0.0.0'):
+		#	message.router_id = RouterID(ip)
+		if message.router_id == RouterID('0.0.0.0'):
+			raise Notify(2,3,'0.0.0.0 is an invalid router_id according to RFC6286')
+		if message.router_id == self.neighbor.router_id and message.asn == self.neighbor.local_as:
+			raise Notify(2,3,'BGP Indendifier collision (%s) on IBGP according to RFC 6286' % message.router_id)
 
 		if message.hold_time < 3:
 			raise Notify(2,6,'Hold Time is invalid (%d)' % message.hold_time)
 		if message.hold_time >= 3:
 			self.neighbor.hold_time = min(self.neighbor.hold_time,message.hold_time)
 
-		if message.router_id == RouterID('0.0.0.0'):
-			message.router_id = RouterID(ip)
+		# XXX: Does not work as the capa is not yet defined
+		if message.capabilities.announced(Capabilities.EXTENDED_MESSAGE):
+			# untested !
+			if self.peer.bgp.message_size:
+				self.message_size = self.peer.bgp.message_size
 
 # README: This limit what we are announcing may cause some issue if you add new family and SIGHUP
 # README: So it is commented until I make my mind to add it or not (as Juniper complain about mismatch capabilities)
@@ -194,50 +218,128 @@ class Protocol (object):
 
 	# Sending message to peer .................................................
 
-	def new_open (self,restarted):
-		o = Open(4,self.neighbor.local_as,self.neighbor.router_id.ip,Capabilities().default(self.neighbor,restarted),self.neighbor.hold_time)
-		self.connection.write(o.message())
+	# we do not buffer those message in purpose
+
+	def new_open (self,restarted,asn4):
+		if asn4:
+			asn = self.neighbor.local_as
+		else:
+			asn = AS_TRANS
+
+		o = Open(4,asn,self.neighbor.router_id.ip,Capabilities().default(self.neighbor,restarted),self.neighbor.hold_time)
+
+		if not self.connection.write(o.message()):
+			raise Failure('Could not send open')
 		return o
 
-	def new_announce (self):
-		asn4 = not not self.peer.open.capabilities.announced(Capabilities.FOUR_BYTES_ASN)
-		# Do not try to join the message and write all in one go as it causes issue if the size is bigger than the MTU
-		# Python 2.5.2 for example send partial data which BGP decoders then take as garbage.
-		count = 0
-		for update in self._delta.announce(asn4,self.neighbor.local_as,self.neighbor.peer_as):
-			count += 1
-			logger.message(self.me(">> UPDATE (update)"))
-			self.connection.write(update)
-			yield count
-
-	def new_eors (self,families):
-		eor = EOR()
-		eors = eor.eors(families)
-		logger.message(self.me(">> UPDATE (eors)"))
-		self.connection.write(eors)
-		return eor.announced()
-
-	def new_update (self):
-		asn4 = not not self.peer.open.capabilities.announced(Capabilities.FOUR_BYTES_ASN)
-		# Do not try to join the message and write all in one go as it causes issue if the size is bigger than the MTU
-		# Python 2.5.2 for example send partial data which BGP decoders then take as garbage.
-		count = 0
-		for update in self._delta.update(asn4,self.neighbor.local_as,self.neighbor.peer_as):
-			count += 1
-			self.connection.write(update)
-			yield count
-
 	def new_keepalive (self,force=False):
 		left = int(self.connection.last_write + self.neighbor.hold_time.keepalive() - time.time())
-		if force or left <= 0:
-			k = KeepAlive()
-			self.connection.write(k.message())
+		k = KeepAlive()
+		m = k.message()
+		if force:
+			written = self.connection.write(k.message())
+			if not written:
+				logger.message(self.me(">> KEEPALIVE buffered"))
+				self._messages[self.neighbor.peer_as].append(('KEEPALIVE',m))
+			else:
+				self._frozen = 0
+			return left,k
+		if left <= 0:
+			written = self.connection.write(k.message())
+			if not written:
+				logger.message(self.me(">> KEEPALIVE buffered"))
+				self._messages[self.neighbor.peer_as].append(('KEEPALIVE',m))
+			else:
+				self._frozen = 0
 			return left,k
 		return left,None
 
 	def new_notification (self,notification):
 		return self.connection.write(notification.message())
 
+	# messages buffered in case of failure
+
+	def buffered (self):
+		return self._messages.get(self.neighbor.peer_as,[]) != []
+
+	def _backlog (self,maximum=0):
+		backlog = self._messages.get(self.neighbor.peer_as,[])
+		if backlog:
+			if not self._frozen:
+				self._frozen = time.time()
+			if self._frozen and self._frozen + (self.neighbor.hold_time) < time.time():
+				raise Failure('peer %s not reading on socket - killing session' % self.neighbor.peer_as)
+			logger.message(self.me("unable to send route for %d second (maximum allowed %d)" % (time.time()-self._frozen,self.neighbor.hold_time)))
+			nb_backlog = len(backlog)
+			if nb_backlog > MAX_BACKLOG:
+				raise Failure('over %d routes buffered for peer %s - killing session' % (MAX_BACKLOG,self.neighbor.peer_as))
+			logger.message(self.me("backlog of %d/%d routes" % (nb_backlog,MAX_BACKLOG)))
+		count = 0
+		while backlog:
+			count += 1
+			name,update = backlog[0]
+			written = self.connection.write(update)
+			if not written:
+				break
+			logger.message(self.me(">> DEBUFFERED %s" % name))
+			backlog.pop(0)
+			self._frozen = 0
+			yield count
+			if maximum and count >= maximum:
+				break
+		self._messages[self.neighbor.peer_as] = backlog
+
+	def _announce (self,name,generator):
+		def chunked (generator,size):
+			chunk = ''
+			for data in generator:
+				if len(data) > size:
+					raise Failure('Can not send BGP update larger than %d bytes on this connection.' % size)
+				if len(chunk) + len(data) <= size:
+					chunk += data
+					continue
+				yield chunk
+				chunk = data
+			if chunk:
+				yield chunk
+
+		# Do not try to join the message and write all in one go as it causes issue if the size is bigger than the MTU
+		# Python 2.5.2 for example send partial data which BGP decoders then take as garbage.
+		count = 0
+		for update in chunked(generator,self.message_size):
+			count += 1
+			if self._messages[self.neighbor.peer_as]:
+				logger.message(self.me(">> %s could not be sent, some messages are still in the buffer" % name))
+				self._messages[self.neighbor.peer_as].append((name,update))
+				continue
+			written = self.connection.write(update)
+			if not written:
+				logger.message(self.me(">> %s buffered" % name))
+				self._messages[self.neighbor.peer_as].append((name,update))
+			yield count
+
+	def new_announce (self):
+		for answer in self._backlog():
+			yield answer
+		asn4 = not not self.peer.open.capabilities.announced(Capabilities.FOUR_BYTES_ASN)
+		for answer in self._announce('UPDATE',self._delta.announce(asn4,self.neighbor.local_as,self.neighbor.peer_as)):
+			yield answer
+
+	def new_update (self):
+		for answer in self._backlog():
+			yield answer
+		asn4 = not not self.peer.open.capabilities.announced(Capabilities.FOUR_BYTES_ASN)
+		for answer in self._announce('UPDATE',self._delta.update(asn4,self.neighbor.local_as,self.neighbor.peer_as)):
+			yield answer
+
+	def new_eors (self,families):
+		for answer in self._backlog():
+			pass
+		eor = EOR()
+		eors = eor.eors(families)
+		for answer in self._announce('EOR',eors):
+			pass
+
 	# Message Factory .................................................
 
 	def KeepAliveFactory (self,data):
@@ -272,7 +374,7 @@ class Protocol (object):
 						# Multiple Capabilities can be present in a single attribute
 						#if r:
 						#	raise Notify(2,0,"Bad length for OPEN %s (size mismatch) %s" % ('capability',hexa(value)))
-	
+
 						if k == Capabilities.MULTIPROTOCOL_EXTENSIONS:
 							if k not in capabilities:
 								capabilities[k] = MultiProtocol()
@@ -280,7 +382,7 @@ class Protocol (object):
 							safi = SAFI(ord(capv[3]))
 							capabilities[k].append((afi,safi))
 							continue
-	
+
 						if k == Capabilities.GRACEFUL_RESTART:
 							restart = unpack('!H',capv[:2])[0]
 							restart_flag = restart >> 12
@@ -295,23 +397,28 @@ class Protocol (object):
 								value_gr = value_gr[4:]
 							capabilities[k] = Graceful(restart_flag,restart_time,families)
 							continue
-	
+
 						if k == Capabilities.FOUR_BYTES_ASN:
 							capabilities[k] = ASN(unpack('!L',capv[:4])[0])
 							continue
-	
+
 						if k == Capabilities.ROUTE_REFRESH:
 							capabilities[k] = RouteRefresh()
 							continue
-	
+
 						if k == Capabilities.CISCO_ROUTE_REFRESH:
 							capabilities[k] = CiscoRouteRefresh()
 							continue
-	
+
+						if k == Capabilities.MULTISESSION_BGP:
+							capabilities[k] = MultiSession()
+							continue
+						if k == Capabilities.MULTISESSION_BGP_OLD:
+							capabilities[k] = MultiSession()
+							continue
+
 						if k not in capabilities:
-							capabilities[k] = Unknown(k)
-						if capv:
-							capabilities[k].append([ord(_) for _ in capv])
+							capabilities[k] = Unknown(k,[ord(_) for _ in capv])
 				else:
 					raise Notify(2,0,'Unknow OPEN parameter %s' % hex(key))
 		return capabilities
@@ -354,7 +461,6 @@ class Protocol (object):
 		attributes = self.AttributesFactory(attribute)
 		routes.extend(self.mp_routes)
 
-		announce = []
 		while announced:
 			nlri = BGPPrefix(AFI.ipv4,announced)
 			route = ReceivedRoute(nlri,'announce')
@@ -378,6 +484,75 @@ class Protocol (object):
 		except IndexError:
 			raise Notify(3,2,data)
 
+	def __new_ASPath (self,data,asn4=False):
+		if asn4:
+			size = 4
+			decoder = 'L' # could it be 'I' as well ?
+		else:
+			size = 2
+			decoder = 'H'
+		stype = ord(data[0])
+		slen = ord(data[1])
+		sdata = data[2:2+(slen*size)]
+
+		ASPS = ASPath(asn4,stype)
+		format = '!'+(decoder*slen)
+		for c in unpack(format,sdata):
+			ASPS.add(c)
+		return ASPS
+
+	def __new_AS4Path (self,data):
+		stype = ord(data[0])
+		slen = ord(data[1])
+		sdata = data[2:2+(slen*4)]
+
+		ASPS = AS4Path(stype)
+		format = '!'+('L'*slen)
+		for c in unpack(format,sdata):
+			ASPS.add(c)
+		return ASPS
+
+	def __merge_attributes (self):
+		as2path = self.attributes[AttributeID.AS_PATH]
+		as4path = self.attributes[AttributeID.AS4_PATH]
+		newASPS = ASPath(True,as2path.asptype)
+		len2 = len(as2path.aspsegment)
+		len4 = len(as4path.aspsegment)
+
+		if len2 < len4:
+			for asn in as4path.aspsegment:
+				newASPS.add(asn)
+		else:
+			for asn in as2path.aspsegment[:-len4]:
+				newASPS.add(asn)
+			for asn in as4path.aspsegment:
+				newASPS.add(asn)
+
+		self.attributes.remove(AttributeID.AS_PATH)
+		self.attributes.remove(AttributeID.AS4_PATH)
+		self.attributes.add(newASPS)
+
+		#raise Notify(3,1,'could not merge AS4_PATH in AS_PATH')
+
+	def __new_communities (self,data):
+		communities = Communities()
+		while data:
+			community = unpack('!L',data[:4])[0]
+			data = data[4:]
+			if data and len(data) < 4:
+				raise Notify(3,1,'could not decode community %s' % str([hex(ord(_)) for _ in data]))
+			communities.add(Community(community))
+		return communities
+
+	def __new_extended_communities (self,data):
+		communities = ECommunities()
+		while data:
+			community = data[:8]
+			data = data[8:]
+			if data and len(data) < 8:
+				raise Notify(3,1,'could not decode extended community %s' % str([hex(ord(_)) for _ in data]))
+			communities.add(ECommunity(community))
+		return communities
 
 	def _AttributesFactory (self,data):
 		if not data:
@@ -399,70 +574,66 @@ class Protocol (object):
 #		if not length:
 #			return self._AttributesFactory(data[length:])
 
+		# XXX: This code does not make sure that attributes are unique - or does it ?
+
 		if code == AttributeID.ORIGIN:
-			#logger.debug('parsing origin')
+			logger.parser('parsing origin')
 			self.attributes.add(Origin(ord(data[0])))
 			return self._AttributesFactory(data[length:])
 
 		if code == AttributeID.AS_PATH:
-			#logger.debug('parsing as_path')
-			def new_ASPath (data,asn4=False):
-				if asn4:
-					size = 4
-					decoder = 'L' # could it be 'I' as well ?
-				else:
-					size = 2
-					decoder = 'H'
-				stype = ord(data[0])
-				slen = ord(data[1])
-				sdata = data[2:2+(slen*size)]
+			logger.parser('parsing as_path')
+			self.attributes.add(self.__new_ASPath(data,self._asn4))
+			if not self._asn4 and self.attributes.has(AttributeID.AS4_PATH):
+				self.__merge_attributes()
+			return self._AttributesFactory(data[length:])
 
-				ASPS = ASPath(stype)
-				for c in unpack('!'+(decoder*slen),sdata):
-					ASPS.add(c)
-				return ASPS
-			self.attributes.add(new_ASPath(data,self._asn4))
+		if code == AttributeID.AS4_PATH:
+			logger.parser('parsing as_path')
+			self.attributes.add(self.__new_AS4Path(data))
+			if not self._asn4 and self.attributes.has(AttributeID.AS_PATH):
+				self.__merge_attributes()
 			return self._AttributesFactory(data[length:])
 
 		if code == AttributeID.NEXT_HOP:
-			#logger.debug('parsing next-hop')
+			logger.parser('parsing next-hop')
 			self.attributes.add(NextHop(Inet(AFI.ipv4,data[:4])))
 			return self._AttributesFactory(data[length:])
 
 		if code == AttributeID.MED:
-			#logger.debug('parsing med')
+			logger.parser('parsing med')
 			self.attributes.add(MED(unpack('!L',data[:4])[0]))
 			return self._AttributesFactory(data[length:])
 
 		if code == AttributeID.LOCAL_PREF:
-			#logger.debug('parsing local-preference')
+			logger.parser('parsing local-preference')
 			self.attributes.add(LocalPreference(unpack('!L',data[:4])[0]))
 			return self._AttributesFactory(data[length:])
 
 		if code == AttributeID.ATOMIC_AGGREGATE:
-			#logger.debug('ignoring atomic-aggregate')
-			# ignore
+			logger.parser('ignoring atomic-aggregate')
 			return self._AttributesFactory(data[length:])
 
 		if code == AttributeID.AGGREGATOR:
-			#logger.debug('ignoring aggregator')
-			# content is 6 bytes
+			logger.parser('ignoring aggregator')
+			return self._AttributesFactory(data[length:])
+
+		if code == AttributeID.AS4_AGGREGATOR:
+			logger.parser('ignoring as4_aggregator')
 			return self._AttributesFactory(data[length:])
 
 		if code == AttributeID.COMMUNITY:
-			#logger.debug('parsing communities')
-			def new_Communities (data):
-				communities = Communities()
-				while data:
-					community = unpack('!L',data[:4])[0]
-					data = data[4:]
-					communities.add(Community(community))
-				return communities
-			self.attributes.add(new_Communities(data[:length]))
+			logger.parser('parsing communities')
+			self.attributes.add(self.__new_communities(data[:length]))
+			return self._AttributesFactory(data[length:])
+
+		if code == AttributeID.EXTENDED_COMMUNITY:
+			logger.parser('parsing communities')
+			self.attributes.add(self.__new_extended_communities(data[:length]))
 			return self._AttributesFactory(data[length:])
 
 		if code == AttributeID.MP_UNREACH_NLRI:
-			#logger.debug('parsing multi-protocol nlri unreacheable')
+			logger.parser('parsing multi-protocol nlri unreacheable')
 			next_attributes = data[length:]
 			data = data[:length]
 			afi,safi = unpack('!HB',data[:3])
@@ -479,7 +650,7 @@ class Protocol (object):
 			return self._AttributesFactory(next_attributes)
 
 		if code == AttributeID.MP_REACH_NLRI:
-			#logger.debug('parsing multi-protocol nlri reacheable')
+			logger.parser('parsing multi-protocol nlri reacheable')
 			next_attributes = data[length:]
 			data = data[:length]
 			afi,safi = unpack('!HB',data[:3])
