@@ -1,14 +1,13 @@
-#!/usr/bin/env python
 # encoding: utf-8
 """
 utils.py
 
 Created by Thomas Mangin on 2009-09-06.
-Copyright (c) 2009-2011 Exa Networks. All rights reserved.
+Copyright (c) 2009-2012 Exa Networks. All rights reserved.
 """
 
-import sys
-import time
+#import sys
+#import time
 
 import StringIO
 import traceback
@@ -24,4 +23,4 @@ def trace ():
 	traceback.print_exc(file=buff)
 	r = buff.getvalue()
 	buff.close()
-	return r
\ No newline at end of file
+	return r
